use std::path::Path;

use s2n_quic::Server;
use tokio::{
    io::{AsyncReadExt, AsyncWriteExt},
    sync::{broadcast, oneshot},
};

use crate::{BUFFER_LENGTH, ServerConfig, voice::play};

pub async fn run<'a>(server_config: ServerConfig<'a>) {
    let mut server = Server::builder()
        .with_io(server_config.server_address)
        .unwrap()
        .with_tls((
            Path::new("certificates/cert.pem"),
            Path::new("certificates/key.pem"),
        ))
        .unwrap()
        .start()
        .unwrap();
    while let Some(mut connection) = server.accept().await {
        println!(
            "Server Name = {}",
            connection.server_name().unwrap().unwrap().to_string()
        );

        println!(
            "Client Address = {}",
            connection.remote_addr().unwrap().to_string()
        );
        let (speaker_sender, speaker_receiver) = broadcast::channel::<f32>(BUFFER_LENGTH);
        let (stop_signal_sender, stop_signal_receiver) = oneshot::channel::<bool>();

        let stream = connection
            .accept_bidirectional_stream()
            .await
            .unwrap()
            .unwrap();

        tokio::spawn(play(speaker_receiver, stop_signal_receiver));

        let (mut receive_stream, send_stream) = stream.split();
        while let Ok(data) = receive_stream.read_f32_le().await {
            speaker_sender.send(data).unwrap();
        }

        return;
    }
}
